#!/usr/bin/env python

# pour affichage
VERT ="\033[1;32m"
BLANC="\033[0;39m"
ROUGE="\033[1;31m"

import os

from scipy import sqrt, integrate, log, floor
from scipy.optimize import fsolve

from mobility import *

from subprocess import call

import numpy as np
import matplotlib.pyplot as plt

OMP_NUM_THREADS=2

Z = 0.5 # fraction de phase initiale
omega_alpha = 0. # saturation phase alpha
omega_beta  = 0.2 # saturation phase beta

# remise a zero du dossier
call(["echo",VERT,"Remise a zero !",BLANC])
call("rm -rf Data*", shell=True)
# definition des parametres
call(["echo",VERT,"Definition des parametres ...",BLANC])
# diffusivites
D1 = 4.7e-19 # [m^2.s-1]
D2 = 5.6e-17 # [m^2.s-1]
# parametres des energies chimiques
k1 = 1.e+11 # [J.m-3]  
k2 = 1.e+9  # [J.m-3]  
a1 = 0.8
a2 = 0.3
b1 = 0.
b2 = 1.*1.e+8
# interface
delta    = 10.e-9 # [m]
gamma    = 1.     # [J.m-2]
w        = 6.*gamma/delta*log(0.95/0.05)
epsilon  = sqrt(3.*gamma*delta/log(0.95/0.05))
mobility = gamma / epsilon**3 * ( D2 * sqrt(2*w) / zeta(k1, k2, a1, a2) )
# temps de simulation
time = 100.*3600. # [s]

###### parametres de calculs
max_D = max(D1, D2)
min_D = min(D1, D2)
# integration temporelle
first_dtime = (delta)**2/max_D
max_dtime   = 1.e+9*first_dtime
min_dtime   = 1.e-3*first_dtime
# longueur du domaine regulier
Lx = 100.*delta # [m]
Ly = 10.*delta  #
Lx_phi = Z*Lx

# normalisation
# choix de trois grandeurs de reference
T = time
L = Lx
E = min(k1, k2)
# raccourci
D = L**2/T
# valeurs normalisees
n_time = time/T
n_max_dtime = max_dtime/T
n_min_dtime = min_dtime/T
n_first_dtime = first_dtime/T
n_Lx = Lx/L
n_Lx_phi = Lx_phi/L
n_Ly = Ly/L
n_delta = delta/L
n_gamma = gamma/(E*L)
n_mobility = mobility*E*T
n_D1 = D1/D
n_D2 = D2/D
n_k1 = k1/E
n_k2 = k2/E
n_b1 = b1/E
n_b2 = b2/E

# lancement du calcul
call(["echo",VERT,"Preparation des fichiers de calcul ...",BLANC])

FOLDER="Data"
call(["mkdir", FOLDER])
call(["cp", "all_increment", FOLDER])

os.chdir(FOLDER)

# ecriture fichier d'entree
call(["../input.py",
      str(n_time),
      str(n_max_dtime),
      str(n_min_dtime),
      str(n_first_dtime),
      str(n_delta),
      str(n_gamma),
      str(n_mobility),
      str(n_D1),
      str(n_D2),
      str(n_k1),
      str(n_k2),
      str(a1),
      str(a2),
      str(n_b1),
      str(n_b2)
      ])
# ecriture du maillage
call("../geof.py "+str(n_Lx)+" "+str(n_Ly)+" "+str(n_delta), shell=True)
# ecriture des champs initiaux

call("../dat.py "+str(n_Lx_phi)+" "+str(n_delta)+" "+str(n_k1)+" "+str(n_k2)+" "+str(a1)+" "+str(a2)+" "+str(b1)+" "+str(b2)+" "+str(omega_alpha)+" "+str(omega_beta), shell=True)

## lancement du calcul
call(["echo",VERT,"Lancement du calcul ...",BLANC])
call(["Zrun","-smp",str(OMP_NUM_THREADS),"input.inp"])

#### lancer post processing
call(["echo",VERT,"Postprocessing ...",BLANC])
call(["./all_increment","input.inp", "DOF_Phi", "DOF_C","NSET_bas"])
call(["Zrun","-pp","-N", str(1),"input.inp"])
call(["Zrun","-pp","-N", str(2),"input.inp"])

### recuperation des calculs
time, phi_av, c1_av = np.loadtxt('input.post', unpack=True, usecols=[0, 1, 2])

c_final = np.loadtxt('Cartes/PP'+str(len(phi_av)), unpack=True, usecols=[4] )

for i in range(1, len(phi_av)):
    print 'plot maps at t', i
    x_plot, phi_plot, c_plot = np.loadtxt('Cartes/PP'+str(i), unpack=True, usecols=[1, 3, 4] )
    
    fig = plt.figure()
    ax1 = plt.subplot2grid((2, 1), (0, 0), title=r'$\phi$')
    ax1.plot(x_plot, phi_plot, color='DodgerBlue', linewidth=2.)
    #ax1.set_xlabel(r'x')
    ax1.set_ylim(0., 1.1)
    
    ax2 = plt.subplot2grid((2, 1), (1, 0), title=r'$c_A$')
    ax2.plot(x_plot, c_plot, color='DarkOrange', linewidth=2.)
    ax2.set_xlabel(r'x')
    ax2.set_ylim(0., 1.)
    
    fig.savefig(str(i)+'.png', format='png')
    plt.close(fig)
    
c_eq_alpha_calcul = c_final[0]
c_eq_beta_calcul  = c_final[-1]

### verification
call(["echo",VERT,"Resultats ...",BLANC])

c_average = Z * (a1+omega_alpha) + (1.-Z) * (a2+omega_beta)

def system(p, arg):
    x = p
    k1, a1, b1, k2, a2, b2, c_average = arg

    y = a2 + k1/k2 * (x - a1)

    f1 = 0.5 * k1 * (x - a1)**2 + b1
    f2 = 0.5 * k2 * (y - a2)**2 + b2
    f1_prime = k1 * (x - a1)
    f2_prime = k2 * (y - a2)
    return f1 + f1_prime * (1.-x) - ( f2 + f2_prime*(1.-y) )


z = fsolve(system, a1, args=([k1, a1, b1, k2, a2, b2, c_average]))[0]
z = [z, a2 + k1/k2 * (z - a1)]

#z = fsolve(system, (a1, a2), args=([k1, a1, b1, k2, a2, b2, c_average]))
z_analytique = (c_average-z[1])/(z[0]-z[1])

print '\n \n'
call(["echo",ROUGE,"conditions initiales ...",BLANC])
print 'concentration \t\t\t\t\t\t\tfraction de phase\t\tconcentration totale'
print 'phase alpha\t\t\tphase beta'
print '--------------------------------------------------------------------------------------------------------------------'
print '\t'+str(a1+omega_alpha)+'\t\t|\t'+str(a2+omega_beta)+'\t\t\t\t\t'+str(Z)+'\t\t\t'+str(c_average)
print '\n \n'
call(["echo",ROUGE,"calcul semi-analytique ...",BLANC])
print 'concentration d\'equilibres\t\t\t\t\tfraction de phase d\'equilibre'
print 'phase alpha\t\t\tphase beta'
print '--------------------------------------------------------------------------------------------------------------------'
print '\t'+str(round(z[0], 3))+'\t\t|\t'+str(round(z[1], 3))+'\t\t\t\t\t'+str(round(z_analytique, 3))+'\t\t\t'+str(z_analytique*z[0]+(1.-z_analytique)*z[1])
print '\n \n'
call(["echo",ROUGE,"calcul numerique ...",BLANC])
print 'concentration d\'equilibres\t\t\t\t\tfraction de phase d\'equilibre'
print 'phase alpha\t\t\tphase beta'
print '--------------------------------------------------------------------------------------------------------------------'
print '\t'+str(round(c_eq_alpha_calcul, 3))+'\t\t|\t'+str(round(c_eq_beta_calcul, 3))+'\t\t\t\t\t'+str(round(phi_av[-1], 3))+'\t\t\t'+str(round(c1_av[-1], 3))
print '\n \n'
