#!/usr/bin/env python

# pour affichage
VERT ="\033[1;32m"
BLANC="\033[0;39m"
ROUGE="\033[1;31m"

import os

from scipy import sqrt, integrate, log, floor
from scipy.optimize import fsolve

from mobility import *

from subprocess import call

import numpy as np

import matplotlib.pyplot as plt
import matplotlib.tri as tri
from matplotlib import cm

OMP_NUM_THREADS=4

Z = 0.5 # fraction de phase initiale
omega_alpha = 0. # saturation phase alpha
omega_beta  = 0.2 # saturation phase beta

# remise a zero du dossier
call(["echo",VERT,"Remise a zero !",BLANC])
call("rm -rf Data*", shell=True)
# definition des parametres
call(["echo",VERT,"Definition des parametres ...",BLANC])
# diffusivites
D1_alpha = 4.7e-19 # [m^2.s-1]
D1_beta  = 5.6e-17 # [m^2.s-1]
D2_alpha = 8.e-18  # [m^2.s-1]
D2_beta  = 5.6e-17 # [m^2.s-1]

# diagramme de phase
k1_alpha = 1.e+11 # [J.m-3]
k1_beta  = 1.e+9  # [J.m-3]
k2_alpha = 1.e+11 # [J.m-3]
k2_beta  = 1.e+11 # [J.m-3]

a1 = 0.8
a2 = 0.
b1 = 0.3
b2 = 0.
# interface
delta    = 10.e-9 # [m]
gamma    = 0.1     # [J.m-2]
w        = 6.*gamma/delta*log(0.95/0.05)
epsilon  = sqrt(3.*gamma*delta/log(0.95/0.05))
mobility = gamma / epsilon**3 * ( D2_alpha * sqrt(2*w) / zeta(k1_alpha, k2_alpha, a1, b1) )

# mecanique
E_alpha = 215.e+9 # [J.m-3]
E_beta = 145.e+9 # [J.m-3]

eps_star = 0.01

n_alpha  = 5.
A_alpha  = 3.5e-20/(10.**(6.*n_alpha)) # [[J.m-3]^{-n}.s^{-1}]
R0_alpha = 100.e+13 # [J.m-3]

n_s_beta   = 5.
K_s_beta   = 1000.e+6 # [J.m-3.s^{1/n}]
tau_0_beta = 65.e+13 # [J.m-3]

Q_beta = 45.e+6 # [J.m-3]
b_beta = 3. # adim
c_beta = 600.e+6 # [J.m-3]
d_beta = 20. # adim

# temps de simulation
time = 2.*3600. # [s]

###### parametres de calculs
max_D = max(D1_alpha, D1_beta, D2_alpha, D2_beta)
min_D = min(D1_alpha, D1_beta, D2_alpha, D2_beta)
# integration temporelle
first_dtime = (delta)**2/max_D
max_dtime   = 2000.*first_dtime
min_dtime   = 0.1*first_dtime
# longueur du domaine regulier
Lx = 25.*delta # [m]
Ly = 25.*delta  #
# normalisation
# choix de trois grandeurs de reference
T = time
L = Lx
E = min(k1_alpha, k2_alpha, k1_beta, k2_beta)
# raccourci
D = L**2/T
# valeurs normalisees
n_time = time/T
n_max_dtime = max_dtime/T
n_min_dtime = min_dtime/T
n_first_dtime = first_dtime/T
n_Lx     = Lx/L
n_Ly     = Ly/L
n_delta  = delta/L
n_gamma  = gamma/(E*L)
n_mobility = mobility*E*T
n_D1_alpha = D1_alpha/D
n_D2_alpha = D2_alpha/D
n_D1_beta = D1_beta/D
n_D2_beta = D2_beta/D
n_k1_alpha = k1_alpha/E
n_k2_alpha = k2_alpha/E
n_k1_beta = k1_beta/E
n_k2_beta = k2_beta/E

n_E_alpha = E_alpha/E
n_E_beta = E_beta/E

n_alpha = 5.
n_A_alpha = A_alpha*E**n_alpha*T
n_R0_alpha = R0_alpha/E

n_s_beta = 5.
n_K_s_beta = K_s_beta/E/(T**(1./n_s_beta))
n_tau_0_beta = tau_0_beta/E

n_Q_beta = Q_beta/E
n_b_beta = b_beta
n_c_beta = c_beta/E
n_d_beta = d_beta
# lancement du calcul
call(["echo",VERT,"Preparation des fichiers de calcul ...",BLANC])

FOLDER="Data"
call(["mkdir", FOLDER])
call("cp octahedral_2D.sys "+str(FOLDER), shell=True)
call(["cp", "all_increment", FOLDER])

os.chdir(FOLDER)

# ecriture fichier d'entree
call(["../input.py",
      str(n_time),
      str(n_max_dtime),
      str(n_min_dtime),
      str(n_first_dtime),
      str(n_delta),
      str(n_gamma),
      str(n_mobility),
      str(n_D1_alpha),
      str(n_D2_alpha),
      str(n_D1_beta),
      str(n_D2_beta),
      str(n_k1_alpha),
      str(n_k2_alpha),
      str(n_k1_beta),
      str(n_k2_beta),
      str(a1),
      str(a2),
      str(b1),
      str(b2),
      str(eps_star),
      str(n_E_alpha),
      str(n_E_beta),
      str(n_alpha),
      str(n_A_alpha),
      str(n_R0_alpha),
      str(n_s_beta),
      str(n_K_s_beta),
      str(n_tau_0_beta),
      str(n_Q_beta),
      str(n_b_beta),
      str(n_c_beta),
      str(n_d_beta)
      ])
# ecriture du maillage
call("../geof.py "+str(n_Lx)+" "+str(n_Ly)+" "+str(n_delta), shell=True)
# ecriture des champs initiaux

call("../dat.py "+str(n_delta)+" "+str(n_k1_alpha)+" "+str(n_k1_beta)+" "+str(a1)+" "+str(a2)+" "+str(b1)+" "+str(b2)+" "+str(omega_alpha)+" "+str(omega_beta), shell=True)

## lancement du calcul
call(["echo",VERT,"Lancement du calcul ...",BLANC])
call(["Zrun","-smp",str(OMP_NUM_THREADS),"input.inp"])

#### lancer post processing
call(["echo",VERT,"Postprocessing ...",BLANC])
call(["./all_increment","input.inp", "DOF_Phi", "DOF_C1", "DOF_sigma11", "DOF_sigma22", "DOF_sigma33", "DOF_sigma12","N_SET_bas"])
call(["Zrun","-pp","-N", str(1),"input.inp"])
call(["Zrun","-pp","-N", str(2),"input.inp"])

### recuperation des calculs
time, phi_av, c1_av = np.loadtxt('input.post', unpack=True, usecols=[0, 1, 2])

c_final = np.loadtxt('Cartes/PP'+str(len(phi_av)), unpack=True, usecols=[4] )

for i in range(1, len(phi_av)):
    print 'plot maps at t', i
    x_plot, y_plot, phi_plot, c_plot, sig11_plot, sig22_plot, sig33_plot, sig12_plot = np.loadtxt('Cartes/PP'+str(i), unpack=True, usecols=[1, 2, 3, 4, 5, 6, 7, 8] )

    triang = tri.Triangulation(x_plot, y_plot)
    
    fig = plt.figure()
    axe = fig.add_subplot(111, aspect='equal')

    plt.tricontour(triang, phi_plot, 128, cmap=plt.cm.Blues) #
    plt.tricontourf(triang, phi_plot, 128, cmap=plt.cm.Blues, extend='both')
    plt.tricontour(x_plot, y_plot, phi_plot, [0.5], colors='Black')
    
    plt.xlim(0., 1.)
    plt.ylim(-0.5, 0.5)

    axe.get_xaxis().set_ticks([])
    axe.get_yaxis().set_ticks([])

    plt.tight_layout()
    
    fig.savefig('phi_'+str(i)+'.png', format='png', bbox_inches='tight')
    plt.close(fig)
    
    fig = plt.figure()
    axe = fig.add_subplot(111, aspect='equal')

    plt.tricontour(triang, c_plot, 128, cmap=plt.cm.YlOrRd) #
    plt.tricontourf(triang, c_plot, 128, cmap=plt.cm.YlOrRd, extend='both')
    plt.tricontour(x_plot, y_plot, phi_plot, [0.5], colors='Black')
    
    plt.xlim(0., 1.)
    plt.ylim(-0.5, 0.5)

    axe.get_xaxis().set_ticks([])
    axe.get_yaxis().set_ticks([])

    plt.tight_layout()
    
    fig.savefig('c_'+str(i)+'.png', format='png', bbox_inches='tight')
    plt.close(fig)

    VM = sqrt(0.5*((sig11_plot-sig22_plot)**2 + (sig22_plot-sig33_plot)**2 + (sig11_plot-sig33_plot)**2 + 6.*(sig12_plot**2)))
    HYDRO = 1./3.*(sig11_plot+sig22_plot+sig33_plot)

    fig = plt.figure()
    axe = fig.add_subplot(111, aspect='equal')

    plt.tricontour(triang, VM, 128, cmap=plt.cm.jet) #
    plt.tricontourf(triang, VM, 128, cmap=plt.cm.jet, extend='both')
    plt.tricontour(x_plot, y_plot, phi_plot, [0.5], colors='Black')
    
    plt.xlim(0., 1.)
    plt.ylim(-0.5, 0.5)
    
    axe.get_xaxis().set_ticks([])
    axe.get_yaxis().set_ticks([])

    plt.tight_layout()
    
    fig.savefig('VM_'+str(i)+'.png', format='png', bbox_inches='tight')
    plt.close(fig)
    
    fig = plt.figure()
    axe = fig.add_subplot(111, aspect='equal')

    plt.tricontour(triang, HYDRO, 128, cmap=plt.cm.jet) #
    plt.tricontourf(triang, HYDRO, 128, cmap=plt.cm.jet, extend='both')
    plt.tricontour(x_plot, y_plot, phi_plot, [0.5], colors='Black')

    plt.xlim(0., 1.)
    plt.ylim(-0.5, 0.5)
    
    axe.get_xaxis().set_ticks([])
    axe.get_yaxis().set_ticks([])

    plt.tight_layout()
    
    fig.savefig('HYDRO_'+str(i)+'.png', format='png', bbox_inches='tight')
    plt.close(fig)
